unit ExtendedListBox;
// ExtendedListBox.pas
{
   tekij: Vesa Tanhua-Tyrkk
    Luotu: 01.12.2001
    Kuvaus:
    Lajennettu TListBox, josta saadaan tapahtuma, kun kyttj valitsee listasta
    jonkun rivin.
    Parannettavaa: Selected ominaisuus toimii vain yhden alkion valinnalle!
    jos valitaan useita alkioita selected on ensimminen. Tosin tmkn ei
    aina toimi!
    Items -property piilotettu, jotta selected ominaisuuden pitminen
    ajantasalla helpompaa.
}
interface

uses
  SysUtils, Classes, Controls, StdCtrls,
    Messages,windows,RTLConsts; //Nm tarvittaan, jotta voidaan lhett viesti, ett on valittu listasta joku kohta katso procedure select
type

TExtendedListBox = class;
  TExtendedListBoxChanged= procedure (sender: TExtendedListBox;Index:integer) of object;

TExtendedListBox = class(TCustomListBox)
  private
    FSelectedIndex:integer;
    FOnAdd: TExtendedListBoxChanged;
    FOnDelete: TExtendedListBoxChanged;
    FSelectedChanged: TExtendedListBoxChanged;
  protected
    property Items;        //kantaluokassa public siirretty, ett saadaan tapahtuma kun listn
    function findSelected():integer;virtual;
    procedure KeyUp(var Key: Word;Shift:TShiftState);override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure SetSelectedIndex(const Value: integer);virtual;
    property MultiSelect;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy; override;
    function GetSelectedIndex: integer;
    function getSelectedText:string;
    procedure Delete(index:Integer);virtual;
    procedure AddItem(text:string;AObject:TObject=nil);override;
    function getItem(index:integer):TObject;overload;virtual;
    procedure DeleteSelected;override;
    procedure MoveSelection(Destination: TCustomListControl);override;
    procedure vapautaObjektit;virtual;
    procedure Select(Index: Integer;Value: Boolean=true);
  published
     property SelectedIndex:integer read GetSelectedIndex write SetSelectedIndex default -1;
     property OnAdd:TExtendedListBoxChanged read FOnAdd write FOnAdd;
     property OnDelete:TExtendedListBoxChanged read FOnDelete write FOnDelete;
     property SelectedChanged:TExtendedListBoxChanged read FSelectedChanged write FSelectedChanged;
     property Style;  property AutoComplete;  property Align;
     property Anchors; property BevelEdges;   property BevelInner;
     property BevelKind default bkNone; property BevelOuter;      property BiDiMode;
     property BorderStyle; property Color; property Columns;
     property Constraints; property Ctl3D; property DragCursor;
     property DragKind; property DragMode;  property Enabled;
     property ExtendedSelect; property Font;property ImeMode;
     property ImeName; property IntegralHeight; property ItemHeight;
     property ParentBiDiMode; property ParentColor;
     property ParentCtl3D; property ParentFont; property ParentShowHint;
     property PopupMenu; property ScrollWidth; property ShowHint;
     property Sorted; property TabOrder; property TabStop;
     property TabWidth; property Visible; property OnClick; property OnContextPopup;
     property OnData; property OnDataFind; property OnDataObject;
     property OnDblClick; property OnDragDrop; property OnDragOver;
     property OnDrawItem; property OnEndDock;  property OnEndDrag;
     property OnEnter; property OnExit; property OnKeyDown; property OnKeyPress;
     property OnKeyUp ; property OnMeasureItem; property OnMouseDown;
     property OnMouseMove; property OnMouseUp; property OnStartDock;
     property OnStartDrag;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TExtendedListBox]);
end;

//**************************************************************
//public metodit
//**************************************************************

//------------------------------------------------------------------------------
// Create(AOwner: TComponent)
// -
//------------------------------------------------------------------------------

constructor TExtendedListBox.Create(AOwner: TComponent);
begin
  inherited;
   SelectedIndex:=-1;
end;

//------------------------------------------------------------------------------
// Destroy
// -
//------------------------------------------------------------------------------

destructor TExtendedListBox.Destroy;
begin
  inherited;
end;

//------------------------------------------------------------------------------
// GetSelectedIndex: integer
// -
//------------------------------------------------------------------------------

function TExtendedListBox.GetSelectedIndex: integer;
begin
  Result:=FSelectedIndex;
end;

//------------------------------------------------------------------------------
// getSelectedText: string
// -
//------------------------------------------------------------------------------

function TExtendedListBox.getSelectedText: string;
begin
  Result:='';
  if SelectedIndex<>-1 then
  Result:=self.Items.Strings[SelectedIndex];
end;
//------------------------------------------------------------------------------
// Delete(index: Integer)
// -
//------------------------------------------------------------------------------

procedure TExtendedListBox.Delete(index: Integer);
begin
    if ((index<0) or (index>=self.Count)) then exit;
    if Assigned(OnDelete) then  self.OnDelete(self,index);
    if self.Items.Objects[index]<>nil then self.Items.Objects[index].Free;
    self.Items.Delete(index);
    SelectedIndex:=findSelected;
end;
 //------------------------------------------------------------------------------
// AddItem(text:string;AObject:TObject=nil)
// -
//------------------------------------------------------------------------------

procedure TExtendedListBox.AddItem(text:string;AObject:TObject=nil);
begin
   if (AObject=nil) then Items.Add(text)
   else Items.AddObject(text,AObject);
   if Assigned(OnAdd) then  OnAdd(self,self.Items.IndexOfObject(AObject));
end;
//------------------------------------------------------------------------------
// getItem(index: integer): TObject
// -
//------------------------------------------------------------------------------

function TExtendedListBox.getItem(index: integer): TObject;
begin
   Result:=nil;
   if((index>=0) and (index<self.Count)) then
   Result:=Items.Objects[index];
end;
//------------------------------------------------------------------------------
// DeleteSelected
// -
//------------------------------------------------------------------------------

procedure TExtendedListBox.DeleteSelected;
  var i:integer;
begin

    if self.SelectedIndex=-1 then exit;
   for  i:=0 to self.Count-1 do
   begin
      if (self.Selected[i])
         then self.Delete(i);
   end;
    SelectedIndex:=findSelected;
end;
 //------------------------------------------------------------------------------
// MoveSelection(Destination: TCustomListControl)
// -
//------------------------------------------------------------------------------

procedure TExtendedListBox.MoveSelection(Destination: TCustomListControl);
    var i:integer;
begin
   i:=0;
   repeat
      if (self.Selected[i]) then begin
       Destination.AddItem(Items.Strings[i],Items.Objects[i]);
       self.Delete(i);
       end
      else i:=i+1;
    until(i=self.Count);
    SelectedIndex:=findSelected;
end;


//------------------------------------------------------------------------------
// vapautaObjektit
// -
//------------------------------------------------------------------------------

procedure TExtendedListBox.vapautaObjektit;
  var I:integer;
  p:TObject;
begin
   for I:=0 to self.Count-1 do begin
      p:=Items.Objects[I];
      p.Free;
   end;
end;
//------------------------------------------------------------------------------
// Select(Index: Integer;Value: Boolean)
// -
//------------------------------------------------------------------------------

procedure TExtendedListBox.Select(Index: Integer;Value: Boolean);
begin
  if MultiSelect then
  begin                             
       SendMessage(Handle, LB_SETSEL, LongInt(false), SelectedIndex);
    if SendMessage(Handle, LB_SETSEL, Longint(Value), Index) = LB_ERR then
      raise EListError.CreateResFmt(@SListIndexError, [Index]);
  end
  else
    if Value then   
    begin
      if SendMessage(Handle, LB_SETCURSEL, Index, 0) = LB_ERR then
        raise EListError.CreateResFmt(@SListIndexError, [Index])
    end
    else
      SendMessage(Handle, LB_SETCURSEL, -1, 0);
    self.SelectedIndex:=Index;
end;

//**************************************************************
//protected metodit
//**************************************************************

//------------------------------------------------------------------------------
// findSelected: integer
// -
//------------------------------------------------------------------------------

function TExtendedListBox.findSelected: integer;
   var i:integer;
begin
    Result:=-1;
   for i:=0 to self.Count-1 do begin
       if (self.Selected[i]) then begin
      Result:=i; exit;end;
   end;
end;
//------------------------------------------------------------------------------
// KeyUp(var Key: Word; Shift: TShiftState)
// -
//------------------------------------------------------------------------------

procedure TExtendedListBox.KeyUp(var Key: Word; Shift: TShiftState);
begin
  inherited;
  SelectedIndex:=findSelected;
end;

//------------------------------------------------------------------------------
// MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer)
// -
//------------------------------------------------------------------------------

procedure TExtendedListBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
  var piste:TPoint;
  index:integer;
begin
  inherited;
  piste.X:=X;
  piste.Y:=y;
  index:=ItemAtPos(piste,true);
  if index<>-1 then  Select(index,true)
  else Select(index,false);
end;
//------------------------------------------------------------------------------
// SetSelectedIndex(const Value: integer)
// -
//------------------------------------------------------------------------------

procedure TExtendedListBox.SetSelectedIndex(const Value: integer);
   var v:Integer;
begin
  v:=FSelectedIndex;
  FSelectedIndex:=Value;
  if v<>FSelectedIndex then
  if Assigned(SelectedChanged) then self.SelectedChanged(self,SelectedIndex);
end;

end.

